/*
   Implementation of a community detection algorithm based on modularity maximization
   including leading eigenvalue bisections, Kernighan-Lin-like fine-tuning, global
   final-tuning, agglomeration tuning, and providing a z-score effect size based on
   an Erdős-Rényi null model. The code also includes two simple proofs of concept (POC).
   Both proofs of concept illustrate how to run the code on a network stored as adjacency
   matrix or adjacency list, with the second POC including an external call to the freely
   available Graphviz software (www.graphviz.org), which produces a visualization of
   the networks with the communities separated by distance and distinguished by colour,
   for increased ease of understanding, using a tuned force-driven spring model.

   Copyright 2012-2015 Charo I. Del Genio

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/








#include"ComDet.h"




/* Initialize the method for use on an adjacency list by allocating memory, copying the degree sequence and compressing the adjacency list */
void modinitlist(int **list, const int *sequence, const int nodes)
{
	int i, j, index, lastused, firstin, hub, maxcomp, queue[nodes];
	
	
	gN = nodes;
	ds     = malloc(nodes*sizeof(int));											// Allocate memory for global degree sequence
	marked = malloc(nodes*sizeof(int));											// and component labels
	gtwom = 0;																	// Initialize global number of stubs
	for (i=0; i<nodes; i++) {
		ds[i] = sequence[i];													// Copy the degree sequence
		gtwom += ds[i];															// and count the global stubs
	}
	compnum  = 0;																// Initialize number of components
	compsize = calloc(nodes,sizeof(int));										// Allocate memory for size of components
	
	for (i=0; i<nodes; i++) marked[i] = -1;										// Determine connected components
	for (i=0; i<nodes; i++) if (!ds[i]) {
		compsize[0]++;
		marked[i] = 0;
	}
	
	compnum = 1;
	for (i=0; i<nodes; i++) {
		if (marked[i]==-1) {
			marked[i] = compnum;
			compsize[compnum]++;
			queue[0] = i;
			lastused = firstin = 0;
			while (lastused>=firstin) {
				hub = queue[firstin++];
				for (j=0; j<ds[hub]; j++) {
					if (marked[list[hub][j]]==-1) {
						marked[list[hub][j]] = compnum;
						queue[++lastused] = list[hub][j];
						compsize[compnum]++;
					}
				}
			}
			compnum++;
		}
	}
	
	ctwom = calloc(compnum,sizeof(long int));									// Allocate memory for component stub numbers
	gadcl = malloc(gtwom*sizeof(int));											// Allocate memory for compressed adjacency matrix
	gadcs = malloc((gN+1)*sizeof(int));
	index = 0;
	for (i=0; i<gN; i++) {
		for (j=0; j<ds[i]; j++) gadcl[index++] = list[i][j];					// Copy the adjacency list into the compressed adjacency matrix
		ctwom[marked[i]] += ds[i];												// Update the component stubs
	}
	gadcs[0]  = 0;																// Record the initial indices in the adjacency matrix
	gadcs[gN] = index;
	for (i=1; i<gN; i++) gadcs[i] = gadcs[i-1]+ds[i-1];
	for (i=0; i<gN; i++) quicksort(gadcl+gadcs[i],ds[i]);						// Sort the compressed adjacency matrix
	globresult.community = malloc(gN*sizeof(int));								// Global community label allocation
	
	cmap    = malloc(gN*sizeof(int));											// Component mapping
	cinvmap = malloc(gN*sizeof(int));											// Component inverse mapping
	maxcomp = 0;																// Prepare memory
	twom = 0;
	for (i=1; i<compnum; i++) {
		if (compsize[i]>maxcomp) maxcomp = compsize[i];
		if (ctwom[i]>twom) twom = ctwom[i];
	}
	if (maxcomp==0) {
		maxcomp = 1;
		twom = 1;
	}
	N = maxcomp;
	prepmem();
	
	
	return;
}




/* Initialize the method for use on an adjacency matrix by allocating memory, extracting the degree sequence and compressing the adjacency matrix */
void modinitmat(int **adj, const int nodes)
{
	int i, j, k, lastused, firstin, hub, maxcomp, queue[nodes];
	
	
	gN = nodes;
	ds     = calloc(nodes,sizeof(int));										// Allocate and initialize global degree sequence
	marked = malloc(nodes*sizeof(int));										// and component labels
	gtwom = 0;																// Initialize global number of stubs
	for (i=0; i<gN; i++) for (j=i+1; j<gN; j++) if (adj[i][j]) {			// Update global degree sequence and number of stubs
		gtwom += 2;
		ds[i]++;
		ds[j]++;
	}
	compnum  = 0;															// Initialize number of components
	compsize = calloc(nodes,sizeof(int));									// Allocate memory for size of components
	
	for (i=0; i<nodes; i++) marked[i] = -1;									// Determine connected components
	for (i=0; i<nodes; i++) if (!ds[i]) {
		compsize[0]++;
		marked[i] = 0;
	}
	
	compnum = 1;
	for (i=0; i<nodes; i++) {
		if (marked[i]==-1) {
			marked[i] = compnum;
			compsize[compnum]++;
			queue[0] = i;
			lastused = firstin = 0;
			while (lastused>=firstin) {
				hub = queue[firstin++];
				j = 0;
				k = 0;
				while (j<ds[hub]) {
					if (adj[hub][k]) {
						j++;
						if (marked[k]==-1) {
							marked[k] = compnum;
							queue[++lastused] = k;
							compsize[compnum]++;
						}
					}
					k++;
				}
			}
			compnum++;
		}
	}
	
	ctwom = calloc(compnum,sizeof(long int));								// Allocate memory for component stub numbers
	for (i=0; i<gN; i++) ctwom[marked[i]] += ds[i];							// Update the component stubs
	gadcl = malloc(gtwom*sizeof(int));										// Allocate memory for compressed adjacency matrix
	gadcs = malloc((gN+1)*sizeof(int));										// Compressed adjacency matrix beginnings vector
	compmat(adj);															// Compress adjacency matrix
	globresult.community = malloc(gN*sizeof(int));							// Global community label allocation
	
	cmap    = malloc(gN*sizeof(int));										// Component mapping
	cinvmap = malloc(gN*sizeof(int));										// Component inverse mapping
	maxcomp = 0;															// Prepare memory
	twom = 0;
	for (i=1; i<compnum; i++) {
		if (compsize[i]>maxcomp) maxcomp = compsize[i];
		if (ctwom[i]>twom) twom = ctwom[i];
	}
	if (maxcomp==0) {
		maxcomp = 1;
		twom = 1;
	}
	N = maxcomp;
	prepmem();
	
	
	return;
}




/* Allocate memory for all needed quantities */
void prepmem(void)
{
	int i;
	long int li;
	
	
	/* Allocate memory */
	result.community = malloc(N*sizeof(int));		// Community label
	adcs             = malloc((N+1)*sizeof(int));	// Compressed component adjacency matrix beginnings vector
	adcl             = malloc(twom*sizeof(int));	// Compressed component adjacency matrix
	seq              = malloc(N*sizeof(int));		// Degree sequence
	comsiz           = malloc(N*sizeof(int));		// Community sizes
	subcl            = malloc(twom*sizeof(int));	// Compressed B matrix
	subcs            = malloc((N+1)*sizeof(int));	// Compressed B matrix beginnings vector
	map              = malloc(N*sizeof(int));		// Node mapping
	invmap           = malloc(N*sizeof(int));		// Node inverse mapping
	subseq           = malloc(N*sizeof(int));		// Degree subsequence
	lead             = malloc(N*sizeof(double));	// Leading eigenvector
	dvdum            = malloc(N*sizeof(double));	// Auxiliary double vectors
	dvdum2           = malloc(N*sizeof(double));
	diags            = malloc(N*sizeof(double));	// Diagonal matrix S
	s                = malloc(N*sizeof(int));		// Community vector s. It is reused as auxiliary array in the agglomeration step.
	s2               = malloc(N*sizeof(int));		// Community vector s2 for Kerninghan and Lin
	ivdum            = malloc(N*sizeof(int));		// Auxiliary integer vectors
	ivdum2           = malloc(N*sizeof(int));
	ivdum3           = malloc(N*sizeof(int));
	ivdumS           = malloc(N*sizeof(int));
	block            = malloc(N*sizeof(int));		// "Blocked" communities
	finalchg         = malloc(N*sizeof(double*));	// Final tuning changes
	finalchg[0]      = malloc(N*N*sizeof(double));
	comlist          = malloc(N*sizeof(int*));		// Community lists
	comlist[0]       = malloc(N*N*sizeof(int));
	finlist          = malloc(N*N*sizeof(int*));	// Final tuning equal list
	finlist[0]       = malloc(2*N*N*sizeof(int));
	for (i=1; i<N; i++) {
		comlist[i]  = comlist[i-1] + N;
		finalchg[i] = finalchg[i-1] + N;
	}
	for (li=1; li<N*N; li++) finlist[li] = finlist[li-1] + 2;
	
	
	return;
}




/* Try to find the network partition that maximizes modularity */
comstruct modularity(double (*rng)(void), const double tol, const double tolpm, const double taccpt)
{
	int i, j, com, comnum, oldcm, index, cindex, flag, split, isthere, component, gcomnum, neg;
	double leadv, dQ, ddQ, dtg2m, finaldQ, aggldQ, expQ, sigmaQ, erprob, e1, e2, s1;
	
	
	toler   = tol;																				// Set tolerances for general purpose and power method
	tolerpm = tolpm;
	tolacc  = taccpt;
	gcomnum = 0;																				// Initialize global community number
	for (component=1; component<compnum; component++) {
		N = compsize[component];
		twom = ctwom[component];
		
		cindex = 0;																				// Build component mappings and degree subsequence
		for (i=0; i<gN; i++) if (marked[i]==component) {
			cmap[i] = cindex;																	// Node i in the original set goes to node cmap[i] for the component submatrix
			seq[cindex] = ds[i];																// Extract degree subsequence
			cinvmap[cindex] = i;																// Node j in the component submatrix is node cinvmap[j] in the original set
			cindex++;
		}
		
		makecomp(component);																	// Extract A matrix from adjacency matrix for current component
		
		for (i=0; i<N; i++) {
			block[i] = 0;																		// Initialize blocked communities
			result.community[i] = 0;															// Initialize community label
			comsiz[i] = 0;																		// Initialize community sizes
		}
		
		comsiz[0] = N;																			// Initial size of first community is N
		comnum = 1;
		do {
			/* Main bisectioning */
			flag = 0;
			oldcm = comnum;
			for (com=0; com<oldcm; com++) {														// For all communities
				if (comsiz[com]>1 && !block[com]) {												// If their size is greater than 1 and they are not blocked
					/* Initialization */
					index = 0;																	// Build node mappings and degree subsequence
					for (i=0; i<N; i++) if (result.community[i]==com) {
							map[i] = index;														// Node i in the component set goes to node map[i] for the community submatrix
							subseq[index] = seq[i];												// Extract degree subsequence
							invmap[index] = i;													// Node j in the community submatrix is node invmap[j] in the component set
							index++;
					}
					makesub(com);																// Extract A' matrix from adjacency matrix for current community
					split = 0;
					
					/* Power method */
					leadv = 0.0;
					dQ = powmtd(comsiz[com],&leadv,&dtg2m,rng);									// Apply power method to community and possibly compute dQ
					comsiz[comnum] = 0;
					idum = comsiz[com];
					if (leadv>0.0) {															// If the leading eigenvalue is positive
						split = 1;
						for (i=0; i<idum; i++) {
							s[i] = s2[i] = lead[i];
							if (lead[i]==-1.0) {
								comsiz[com]--;
								comsiz[comnum]++;
								result.community[invmap[i]] = comnum;
							}
						}
					} else {																	// Otherwise
						for (i=0; i<idum; i++) s[i] = s2[i] = 1;								// leave all the nodes in their original community
						dQ = 0.0;																// and put the increase of modularity to 0
					}
					
					/* Fine tuning via Kerninhan-Lin algorithm */
					ddQ = KLin(rng);															// Perform fine tuning and check the potential new modularity
					if (ddQ > tolacc && ddQ + dQ > tolacc) {									// If the fine tuning plus the potential power method split would increase the modularity
						split = 1;
						for (i=0; i<idum; i++) {												// Then for all nodes
							if (s[i]==-1 && s2[i]==1) {											// check if they need to be brought back from the second community to the first
								comsiz[comnum]--;
								comsiz[com]++;
								result.community[invmap[i]] = com;
							} else if (s[i]==1 && s2[i]==-1) {									// or if they need to be moved from the first community to the second
								comsiz[com]--;
								comsiz[comnum]++;
								result.community[invmap[i]] = comnum;
							}
						}
					}
					
					/* Verification that power method and fine tuning worked properly */
					if (split && (comsiz[comnum]==0 || comsiz[com]==0 || ( (ddQ<=tolacc || ddQ+dQ<=tolacc) && dQ<0.0 ))) {
						split = 0;																// If the two steps above resulted in one community being empty
						if (comsiz[com]==0 || ( (ddQ<=tolacc || ddQ+dQ<=tolacc) && dQ<0.0 )) {	// or if the KL failed and the original guess from the power method
							comsiz[com] = idum;													// was actually decreasing modularity,
							comsiz[comnum] = 0;													// then put all the nodes in the first community,
							for (j=0; j<idum; j++) result.community[invmap[j]] = com;			// and behave like nothing happened.
						}
					}
					if (split) {
						comnum++;																// If a split was somehow done, increase the number of communities
						flag = 1;
					} else block[com] = 1;														// Otherwise mark the original community as blocked
				}
			}
			
			/* Final tuning */
			for (i=0; i<N; i++) s[i] = result.community[i];
			oldcm = comnum;
			finaldQ = final(&comnum,rng);														// Perform final tuning and check the potential new modularity
			if (finaldQ > tolacc) {																// If final tuning could increase modularity
				flag = 1;
				for (i=0; i<N; i++) {															// Perform node switches as requested
					result.community[i] = s[i];													// unblock any blocked communities
					block[i] = 0;
				}
			} else {																			// Otherwise
				comnum = oldcm;																	// Just restore the correct pieces of information
				for (i=0; i<comnum; i++) comsiz[i] = 0;
				for (i=0; i<N; i++) comlist[result.community[i]][comsiz[result.community[i]]++] = i;
			}
			
			/* Agglomeration */
			if (comnum>1) {																		// If there is more than 1 community
				oldcm = comnum;
				for (i=0; i<N; i++) ivdumS[i] = result.community[i];
				aggldQ = agglom(&comnum,rng);													// try an agglomeration step
				if (aggldQ >= tolacc) {															// If an agglomeration would increase modularity
					flag = 1;
					for (i=0; i<N; i++) {														// Perform the node switches as required
						block[i] = 0;
						result.community[i] = ivdumS[i];
					}
				} else {																		// Otherwise
					comnum = oldcm;																// Just restore the correct pieces of information
					for (i=0; i<comnum; i++) comsiz[i] = 0;
					for (i=0; i<N; i++) comlist[result.community[i]][comsiz[result.community[i]]++] = i;
				}
			}
		} while (flag);
		
		for (i=0; i<N; i++) globresult.community[cinvmap[i]]=result.community[i]+gcomnum;
		gcomnum += comnum;
	}
	
	if (compsize[0]) {
		for (i=0; i<gN; i++) if (marked[i]==0) globresult.community[i]=gcomnum;
		gcomnum++;
	}
	
	/* Compute modularity and z-score */
	globresult.modularity = 0.0;
	for (i=0; i<gN; i++) for (j=0; j<gN; j++) if (globresult.community[i]==globresult.community[j]) {
		isthere = 0;
		index = gadcs[i];
		while (index<gadcs[i+1] && !isthere) if (gadcl[index++]==j) isthere = 1;
		globresult.modularity += (((double) isthere) - ((double) (ds[i]*ds[j]))/((double) gtwom))/((double) gtwom);
	}
	erprob = log(gtwom)-log(gN)-log(gN-1);
	e1 = log1p(-(7.0/5.0)*exp(-((double) gN)/50.0)) + log(0.97) + 0.5*(log1p(-exp(erprob))-log(gN)-erprob);
	e2 = -(1.0/6.0)*log(gN*(2.0/5.0))*erprob+(5.0/4.0)*log1p(-exp(erprob))+((13.0/15.0)*exp(-((double) gN)/100.0)-(6.0/5.0))*log(gN);
	expQ = fmax(e1,e2) + log1p(exp(-fabs(e1-e2)));
	if (expQ>=0.0) expQ=log(3);
	else expQ = log(2) + log1p(exp(expQ-log(2)));
	s1 = 1.0-exp(-((double) (gN-10))/50.0);
	s1 = log1p(s1);
	sigmaQ = 0.5*(s1 + 2.0*log(0.97) - log(2) - 3.0*log(gN) - 2.0*erprob);
	if (log1p(globresult.modularity+1.0)>=expQ) neg=1;
	else neg=-1;
	globresult.zscore = neg*exp(fmax(log1p(globresult.modularity+1.0),expQ) + log1p(-exp(-fabs(log1p(globresult.modularity+1.0)-expQ))) - sigmaQ);
	
	
	return globresult;
}




/* Free memory used */
void modclean(void)
{
	free(finalchg[0]);
	free(finalchg);
	free(comlist[0]);
	free(comlist);
	free(finlist[0]);
	free(finlist);
	free(block);
	free(ivdumS);
	free(ivdum3);
	free(ivdum2);
	free(ivdum);
	free(s2);
	free(s);
	free(diags);
	free(dvdum2);
	free(dvdum);
	free(lead);
	free(subseq);
	free(cmap);
	free(cinvmap);
	free(map);
	free(invmap);
	free(subcl);
	free(subcs);
	free(comsiz);
	free(result.community);
	free(seq);
	free(adcs);
	free(adcl);
	free(globresult.community);
	free(ds);
	free(marked);
	free(compsize);
	free(ctwom);
	free(gadcs);
	free(gadcl);
	
	
	return;
}




/* Agglomeration */
double agglom(int *comnum, double (*rng)(void))
{
	int one, two, i, j, done, flag, isthere, index;
	long int multi, ext;
	double max, res=0.0;
	
	
	/* ivdumS records the target communities suggested by the final tuning; in the beginning it contains the original communities of the nodes
	   dvdum2 contains the total modularity increase if accepting agglomerations up to the correspondent element
	   and it is reused for the intermediate step, ivdum contains used community labels, ivdum2 contains the lower-index community of a merged pair
	   ivdum3 contains the higher-index community of a merged pair. The array s is used here to store the indices of different intermediate
	   steps corresponding to the same modularity increase */
	do {
		/* Clean empty communities if there are any */
		do {
			flag=0;
			for (i=0; i<(*comnum); i++) comsiz[i] = 0;
			for (i=0; i<N; i++) comlist[ivdumS[i]][comsiz[ivdumS[i]]++] = i;
			i = 0;
			while (i<(*comnum)) {
				if (!comsiz[i]) {
					flag = 1;
					(*comnum)--;
					for (j=i; j<(*comnum); j++) comsiz[j] = comsiz[j+1];
					for (j=0; j<N; j++) if (ivdumS[j]>i) ivdumS[j]--;
				}
				else i++;
			}
		} while (flag);
		
		/* If there is more than one community */
		if ((*comnum)>1) {
			/* Build the matrix containing the initial change in modularity */
			for (one=0; one<(*comnum)-1; one++) {
				for (two=one+1; two<(*comnum); two++) {
					finalchg[one][two] = 0.0;
					for (i=0; i<comsiz[one]; i++) for (j=0; j<comsiz[two]; j++) {
						isthere = 0;
						index = adcs[comlist[one][i]];
						while (index<adcs[comlist[one][i]+1] && !isthere) if (adcl[index++]==comlist[two][j]) isthere = 1;
						finalchg[one][two] += (2.0*((double) isthere)/((double) twom)-2.0*((double) (seq[comlist[one][i]]*seq[comlist[two][j]]))/((double) (twom*twom)));
					}
				}
				ivdum[one] = 0;
			}
			ivdum[(*comnum)-1] = 0;
			flag = 0;
			
			/* Find the two communities whose aggregate gives the greatest increase in modularity */
			max = finalchg[0][1];
			finlist[0][0] = 0;
			finlist[0][1] = 1;
			for (one=0; one<(*comnum)-1; one++) for (two=one+1; two<(*comnum); two++) if (finalchg[one][two]>max) max=finalchg[one][two];
			
			/* If more than one pair of communities yields the same modularity increase, pick one pair randomly */
			multi = 0;
			for (one=0; one<(*comnum)-1; one++) for (two=one+1; two<(*comnum); two++) if (max-finalchg[one][two]<toler) {
				finlist[multi][0] = one;
				finlist[multi++][1] = two;
			}
			ext = ((long int) (rng()*multi));
			
			ivdum2[0] = finlist[ext][0];																									// Record the incremental increase in modularity
			ivdum3[0] = finlist[ext][1];
			dvdum2[0] = finalchg[ivdum2[0]][ivdum3[0]];																						// and the choice done
			ivdum[finlist[ext][1]] = 1;																										// Mark the higher-index community as merged
			
			for (done=1; done<(*comnum)-1; done++) {																						// For all the other choices
				/* Efficiently update the modularity increase matrix */
				for (two=ivdum2[done-1]+1; two<(*comnum); two++) if (!ivdum[two]) finalchg[ivdum2[done-1]][two] += finalchg[(two<ivdum3[done-1] ? two : ivdum3[done-1])][(two<ivdum3[done-1] ? ivdum3[done-1] : two)];
				for (one=0; one<ivdum2[done-1]; one++) if (!ivdum[one]) finalchg[one][ivdum2[done-1]] += finalchg[(one<ivdum3[done-1] ? one : ivdum3[done-1])][(one<ivdum3[done-1] ? ivdum3[done-1] : one)];
				
				/* Find the two communities whose aggregate gives the greatest increase in modularity */
				i = 0;
				while (ivdum[i]) i++;
				j = i+1;
				while (ivdum[j]) j++;
				max = finalchg[i][j];
				finlist[done][0] = i;
				finlist[done][1] = j;
				for (one=i; one<(*comnum)-1; one++) if (!ivdum[one]) for (two=one+1; two<(*comnum); two++) if (!ivdum[two] && finalchg[one][two]>max) max=finalchg[one][two];
				
				/* If more than one pair of communities yields the same modularity increase, pick one pair randomly */
				multi = 0;
				for (one=i; one<(*comnum)-1; one++) if (!ivdum[one]) for (two=one+1; two<(*comnum); two++) if (!ivdum[two] && fabs(finalchg[one][two]-max)<toler) {
					finlist[multi][0] = one;
					finlist[multi++][1] = two;
				}
				ext = ((long int) (rng()*multi));
				
				ivdum2[done] = finlist[ext][0];																								// Record the incremental increase in modularity
				ivdum3[done] = finlist[ext][1];
				dvdum2[done] = dvdum2[done-1]+finalchg[ivdum2[done]][ivdum3[done]];															// and the choice done
				ivdum[finlist[ext][1]] = 1;																									// Mark the second community as merged
			}
			
			max = dvdum2[0];																												// Find the intermediate step
			ext = 0;																														// yielding the highest increase in modularity
			for (i=1; i<(*comnum)-1; i++) if (dvdum2[i]>max) max=dvdum2[i];
			
			if (max>=toler) {																												// If modularity would increase. If more than one
				multi = 0;																													// intermediate step corresponds to the maximum
				for (i=0; i<(*comnum)-1; i++) if (max-dvdum2[i]<toler) ext=i;																// increase in modularity, pick the most agglomerated
				res += dvdum2[ext];																											// Update the total accepted modularity increase
				for (j=0; j<=ext; j++) {																									// Agglomerate the communities
					for (i=0; i<comsiz[ivdum3[j]]; i++) {
						ivdumS[comlist[ivdum3[j]][i]] = ivdum2[j];
						comlist[ivdum2[j]][i+comsiz[ivdum2[j]]] = comlist[ivdum3[j]][i];
					}
					comsiz[ivdum2[j]] += comsiz[ivdum3[j]];
					comsiz[ivdum3[j]] = 0;
				}
				flag = 1;																													// Flag that an improvement was found
			}
		}
	} while (flag && (*comnum)>1);																											// Repeat as long as possible if modularity increases
	
	
	return res;
}




/* Final tuning */
double final(int *comnum, double (*rng)(void))
{
	int i, j, k, maxn, maxto, flag, limit, oldlm, oldcm, isthere, index;
	long int multi, ext;
	double equalpiece, max, res=0.0;
	
	
	/* ivdum contains used node labels, ivdum2 contains the sequence of the nodes to move
	   dvdum2 contains the total modularity increase if accepting moves up to the correspondent element
	   and it is reused for the intermediate step
	   ivdumS contains the community number to which the corresponding node is moved
	   The array s is used to record the target communities suggested by the final tuning */
	if (*comnum<N) limit=*comnum;																									// Define number of communities to consider initially
	else limit=(*comnum)-1;
	
	do {
		/* Clean empty communities if there are any */

		do {
			flag=0;
			for (i=0; i<=limit; i++) comsiz[i] = 0;
			for (i=0; i<N; i++) {
				comlist[s[i]][comsiz[s[i]]] = i;
				comsiz[s[i]]++;
			}
			i = 0;
			while (i<limit) {
				if (!comsiz[i]) {
					flag = 1;
					limit--;
					(*comnum)--;
					for (j=i; j<=limit; j++) comsiz[j] = comsiz[j+1];
					for (j=0; j<N; j++) if (s[j]>=i) s[j]--;
				}
				else i++;
			}
		} while (flag);

		flag = 0;
		oldlm = limit;																												// Record old values of cycle limit
		oldcm = *comnum;																											// and number of communities
		for (i=0; i<N; i++) {																										// Compute initial ddQ matrix
			equalpiece = -2.0*((double) (seq[i]*seq[i]))/((double) (twom*twom));
			for (j=adcs[i]; j<adcs[i+1]; j++) if (s[adcl[j]]==s[i]) equalpiece -= 2.0/((double) twom);
			for (j=0; j<comsiz[s[i]]; j++) equalpiece += 2.0*((double) (seq[i]*seq[comlist[s[i]][j]]))/((double) (twom*twom));
			for (j=0; j<=limit; j++) finalchg[i][j] = equalpiece;
			for (j=adcs[i]; j<adcs[i+1]; j++) finalchg[i][s[adcl[j]]] += 2.0/((double) twom);
			for (j=0; j<N; j++) finalchg[i][s[j]] -= 2.0*((double) (seq[i]*seq[j]))/((double) (twom*twom));
			ivdum[i] = 0;
		}
		
		i = 0;																														// Find node and community that will give
		if (s[0]==0) j = 1;																											// greatest increase in modularity
		else j = 0;																													// when moving the node into the community
		max = finalchg[0][j];
		j++;
		while (i<N) {
			while (j<=limit) {
				if (s[i]!=j && finalchg[i][j]>max) max = finalchg[i][j];
				j++;
			}
			i++;
			j = 0;
		}
		
		multi = 0;																													// If more than one move
		for (i=0; i<N; i++) for (j=0; j<=limit; j++) if (s[i]!=j && max-finalchg[i][j]<toler) {										// gives the same maximum modularity increase
			finlist[multi][0] = i;																									// then pick one randomly
			finlist[multi++][1] = j;
		}
		ext = ((long int) (rng()*multi));
		maxn  = finlist[ext][0];
		maxto = finlist[ext][1];
		
		ivdum[maxn] = 1;																											// Mark the node as used
		dvdum2[0] = finalchg[maxn][maxto];																							// record the choice
		ivdum2[0] = maxn;																											// and the incremental increase in modularity
		ivdumS[0] = maxto;
		if (maxto==limit && limit<N-1) {																							// If the node is being moved into an empty community
			limit++;																												// increase the number of communities
			(*comnum)++;
			for (i=0; i<N; i++) finalchg[i][limit]=finalchg[i][limit-1];															// and extend the ddQ matrix
		}
		
		
		for (j=1; j<N; j++) {																										// For all the other choices
			for (i=0; i<N; i++) if (!ivdum[i]) {																					// Update the ddQ matrix
				isthere = 0;
				index = adcs[maxn];
				while (index<adcs[maxn+1] && !isthere) if (adcl[index++]==i) isthere = 1;
				equalpiece = 2.0*((double) isthere)/((double) twom) - 2.0*((double) (seq[i]*seq[maxn]))/((double) (twom*twom));
				finalchg[i][s[maxn]] -= equalpiece;
				finalchg[i][maxto] += equalpiece;
				if (s[i]==s[maxn]) for (k=0; k<=limit; k++) finalchg[i][k] += equalpiece;
				else if (s[i]==maxto) for (k=0; k<=limit; k++) finalchg[i][k] -= equalpiece;
			}
			
			i = 0;																													// Find best node and community and proceed as before
			while (ivdum[i]) i++;
			if (s[i]==0) k = 1;
			else k = 0;
			max = finalchg[i][k];
			k++;
			while (i<N) {
				if (!ivdum[i]) {
					while (k<=limit) {
						if (s[i]!=k && finalchg[i][k]>max) max = finalchg[i][k];
						k++;
					}
				}
				i++;
				k = 0;
			}
			
			multi = 0;
			for (i=0; i<N; i++) if (!ivdum[i]) for (k=0; k<=limit; k++) if (s[i]!=k && max-finalchg[i][k]<toler) {
				finlist[multi][0] = i;
				finlist[multi++][1] = k;
			}
			ext = ((long int) (rng()*multi));
			maxn  = finlist[ext][0];
			maxto = finlist[ext][1];
			
			ivdum[maxn] = 1;
			dvdum2[j] = dvdum2[j-1] + finalchg[maxn][maxto];
			ivdum2[j] = maxn;
			ivdumS[j] = maxto;
			if (maxto==limit && limit<N-1) {
				limit++;
				(*comnum)++;
				for (i=0; i<N; i++) finalchg[i][limit]=finalchg[i][limit-1];
			}
		}
		
		max = dvdum2[0];																											// Find best intermediate point
		for (i=1; i<N; i++) if (dvdum2[i]>max) max = dvdum2[i];
		
		limit = oldlm;																												// Revert values of cycle limit
		*comnum = oldcm;																											// and number of communities to what they were at the beginning
		if (max>toler) {																											// If modularity would increase
			multi = 0;																												// If more than one intermediate point
			for (i=0; i<N; i++) if (max-dvdum2[i]<toler) finlist[multi++][0] = i;													// gives the same maximum modularity increase
			maxn = finlist[((int) (rng()*multi))][0];																				// pick one randomly
			for (i=0; i<=maxn; i++) {
				s[ivdum2[i]] = ivdumS[i];																							// Use intermediate point as new starting configuration
				if (ivdumS[i] == limit && limit < N-1) {
					limit++;
					(*comnum)++;
				}
			}
			res += dvdum2[maxn];																									// Update the total accepted modularity increase
			flag = 1;																												// Flag that an improvement was found
		}
	} while (flag);																													// Repeat as long as modularity would increase
	
	
	return res;
}




/* Kernighan-Lin algorithm */
double KLin(double (*rng)(void))
{
	int i, j, maxn, flag, multi;
	double max, res=0.0, dtx2m;
	
	
	/* dvdum contains the increase in modularity given by the correspondent elements, ivdum contains labels for used nodes
	   dvdum2 contains the total modularity increase if accepting switches up to the correspondent element
	   and it is reused for the intermediate step. ivdum2 contains the sequence of the nodes to switch
	   ivdumS contains a set of node indices all corresponding to the maximum modularity increase */
	do {
		dtx2m = 0.0;																								// Dot product of subsequence vector and label vector
		for (j=0; j<idum; j++) dtx2m += subseq[j]*s2[j];
		for (i=0; i<idum; i++) {																					// Build vector of changes d(dQ)
			dvdum[i] = 0.0;
			for (j=subcs[i]; j<subcs[i+1]; j++) dvdum[i] += s2[subcl[j]]*(2.0/twom);
			dvdum[i] -= subseq[i]*dtx2m*(2.0/(twom*twom));
			dvdum[i] -= diags[i]*s2[i]*(2.0/twom);
			dvdum[i] *= -s2[i];
			dvdum[i] -= subseq[i]*subseq[i]*(2.0/(twom*twom)) + diags[i]*(2.0/twom);
			ivdum[i] = 0;
		}
		
		flag = 0;
		max = dvdum[0];																								// Find element of max change
		for (i=1; i<idum; i++) if (dvdum[i]>max) max = dvdum[i];
		
		multi = 0;																									// If there are more than 1 element yielding the max increase
		for (i=0; i<idum; i++) if (max-dvdum[i]<toler) ivdumS[multi++] = i;											// pick one randomly
		maxn = ivdumS[((int) (rng()*multi))];
		
		ivdum[maxn] = 1;																							// Identify node as used
		dvdum2[0] = dvdum[maxn];																					// Store modularity increase
		ivdum2[0] = maxn;																							// Store switched node
		
		for (j=1; j<idum; j++) {																					// For all other needed choices
			for (i=subcs[maxn]; i<subcs[maxn+1]; i++) dvdum[subcl[i]] += s2[subcl[i]] * s2[maxn] * (4.0/twom);		// Update the vector of changes d(dQ)
			for (i=0; i<idum; i++) dvdum[i] -= s2[i] * subseq[i] * subseq[maxn] * s2[maxn] * (4.0/(twom*twom));
			dvdum[maxn] -= s2[maxn] * s2[maxn] * diags[maxn] * (4.0/twom);
			
			maxn = 0;
			while (ivdum[maxn]) maxn++;																				// Find new element of max change
			max = dvdum[maxn];																						// Making sure it was not already used
			for (i=maxn+1; i<idum; i++) if (!ivdum[i] && dvdum[i]>max) max = dvdum[i];
			
			multi = 0;																								// If there are more than 1 element yielding the max increase
			for (i=0; i<idum; i++) if (!ivdum[i] && max-dvdum[i]<toler) ivdumS[multi++] = i;						// pick one randomly
			maxn = ivdumS[((int) (rng()*multi))];
			
			ivdum[maxn] = 1;																						// Identify node as used
			dvdum2[j] = dvdum2[j-1] + dvdum[maxn];																	// Store modularity increase
			ivdum2[j] = maxn;																						// Store switched node
		}
		
		max = dvdum2[0];																							// Find point of max modularity increase in sequence of swaps
		for (i=1; i<idum; i++) if (dvdum2[i]>max) max = dvdum2[i];
		
		if (max>toler) {																							// If the maximum modularity increase is greater than 0
			multi = 0;																								// If more than once during the sequence the max modularity increase
			for (i=0; i<idum; i++) if (max-dvdum2[i]<toler) ivdumS[multi++] = i;									// was reached, pick one point randomly
			maxn = ivdumS[((int) (rng()*multi))];
			
			res += dvdum2[maxn];																					// Update the total accepted modularity increase
			for (i=0; i<=maxn; i++) s2[ivdum2[i]] *= -1;															// Switch the relevant nodes' communities
			flag = 1;																								// Flag that an improvement was found
		}
	} while (flag);																									// Repeat the cycle as long as improvements are found
	
	
	return res;
}




/* Apply power method to community and compute the corresponding dQ if possible */
double powmtd(const int size, double *leadv, double *dtg2m, double (*rng)(void))
{
	int i, j, iter, limit, flag=0, goahead, ok;
	double dtx2m, oldv, dotp=1.0, x=0.0, res=0.0;
	
	
	*dtg2m = 0.0;
	for (i=0; i<size; i++) *dtg2m += ((double) subseq[i])/((double) twom);						// Compute sum of degree subsequence divided by 2m
	for (i=0; i<size; i++) diags[i] = subcs[i+1]-subcs[i] - (*dtg2m)*subseq[i];					// Compute elements of diagonal matrix S
	limit = 1000;
	
	/* Power method */
	do {
		if (dotp<0.0 || *leadv-x<0.0) {
			flag = 1;
			x = *leadv-x;
		}
		do {
			for (i=0; i<size; i++) lead[i] = 2.0*rng()-1.0;									// Build initial guess for leading eigenvector
			ok = 0;
			*leadv = 0.0;
			dtx2m = 0.0;
			for (i=0; i<size; i++) dtx2m += (lead[i]*subseq[i])/twom;
			for (i=0; i<size; i++) {
				dvdum[i] = 0.0;
				if (subcs[i+1]!=subcs[i]) ok=1;
				for (j=subcs[i]; j<subcs[i+1]; j++) dvdum[i] += lead[subcl[j]];
				dvdum[i] -= dtx2m*subseq[i] + (diags[i]-x)*lead[i];
				*leadv += pow(dvdum[i],2.0);
			}
			*leadv = sqrt(*leadv);
		} while (*leadv == 0.0 && ok);
		
		if (ok) {
			*leadv = 1.0;
			iter = 0;
			do {
				iter++;
				if (iter>limit) {
					if (fabs(*leadv)/10 > 0.1 && fabs(*leadv)/10 < 0.5)	x += fabs(*leadv)/10;
					else x += 0.5;
					iter = 1;
				}
				dtx2m = 0.0;
				oldv = *leadv;
				for (i=0; i<size; i++) dtx2m += (lead[i]*subseq[i])/(oldv*twom);				// Compute dot product of degree subsequence and current guess for leading eigenvector divided by 2m
				
				*leadv = 0.0;
				for (i=0; i<size; i++) {
					dvdum[i] = 0.0;
					for (j=subcs[i]; j<subcs[i+1]; j++) dvdum[i] += lead[subcl[j]]/oldv;
					dvdum[i] -= dtx2m*subseq[i] + (diags[i]-x)*lead[i]/oldv;
					*leadv += pow(dvdum[i],2.0);
				}
				*leadv = sqrt(*leadv);
			 	
				dotp = 0.0;
				for (i=0; i<size; i++) {
					dotp += dvdum[i]*lead[i];
					lead[i] = dvdum[i];
				}
				
				goahead = 1;
				if (oldv!=0.0 && (*leadv)!=0.0 && iter!=1 && 1.0-fabs(dotp/(oldv*(*leadv)))<=tolerpm && fabs(*leadv-oldv)<=tolerpm) goahead=0;
			} while (goahead);
		} else {
			for (i=0; i<size; i++) lead[i]=0.0;
			*leadv = 0.0;
			x = 0.0;
		}
	} while (ok && (dotp<0.0 || *leadv-x<0.0) && !flag);
	
	*leadv -= x;
	
	dtx2m = 0.0;
	for (i=0; i<size; i++) {
		if (lead[i]>0.0) lead[i]=1.0;
		else if (lead[i]<0.0) lead[i]=-1.0;
		else if (rng()<0.5) lead[i]=-1.0;
		else lead[i]=1.0;
		dtx2m += lead[i]*(((double) subseq[i])/((double) twom));
	}
	for (i=0; i<size; i++) {
		dvdum[i] = 0.0;
		for (j=subcs[i]; j<subcs[i+1]; j++) dvdum[i] += lead[subcl[j]];
		dvdum[i] -= dtx2m*subseq[i] + diags[i]*lead[i];
		res += 0.5*lead[i]*dvdum[i]/twom;
	}
	
	
	return res;
}




/* Extract B matrix */
void makesub(const int com)
{
	int i, j, Bindex=0, flag=0, last;
	
	
	/* Works like compmat. To understand it, see the comments in there. */
	for (i=0; i<N; i++) if (result.community[i]==com) {
		flag = 0;
		last = map[i];
		subcs[last] = Bindex;
		for (j=adcs[i]; j<adcs[i+1]; j++) if (result.community[adcl[j]]==com) {
			if (!flag) {
				subcs[last] = Bindex;
				flag = 1;
			}
			subcl[Bindex++] = map[adcl[j]];
		}
	}
	subcs[last+1] = Bindex;
	
	
	return;
}




/* Extract component matrix */
void makecomp(const int component)
{
	int i, j, Cindex=0, flag=0, last;
	
	
	/* Works like compmat. To understand it, see the comments in there. */
	for (i=0; i<gN; i++) if (marked[i]==component) {
		flag = 0;
		last = cmap[i];
		adcs[last] = Cindex;
		for (j=gadcs[i]; j<gadcs[i+1]; j++) if (marked[gadcl[j]]==component) {
			if (!flag) {
				adcs[last] = Cindex;
				flag = 1;
			}
			adcl[Cindex++] = cmap[gadcl[j]];
		}
	}
	adcs[last+1] = Cindex;
	
	
	return;
}




/* Compress adjacency matrix */
void compmat(int **adj)
{
	int i, j, flag, index=0;
	
	
	for (i=0; i<gN; i++) {				// For all the rows of the adjacency matrix
		flag = 0;						// Flag indicates if the first non-zero element of the row has been found
		gadcs[i] = index;				// Initially consider the first available position as the beginning of the non-zero column list
		for (j=0; j<gN; j++) {			// For all the columns
			if (adj[i][j]) {			// If there is a link between i and j
				if (!flag) {
					gadcs[i] = index;	// Update the position of the first non-zero element if needed
					flag = 1;
				}
				gadcl[index++] = j;		// And record the column number
			}
		}
	}
	gadcs[gN] = index;					// Position after the last element
	
	
	return;
}




/* Sorts the sequence in non-increasing order */
void quicksort(int *s, const int length)
{
	int i, anyone, pivot, l_ind=0, s_ind=0, h_ind=0, *lower, *higher, *same;
	
	if (length<=1) return;
	
	lower  = malloc(length*sizeof(int));
	higher = malloc(length*sizeof(int));
	same   = malloc(length*sizeof(int));
	
	anyone = (int) (floor ( ( ( (double) rand() ) / ( (double) RAND_MAX ) ) * (double) length ));
	pivot  = s[anyone];
	
	for (i=0; i<length; i++) {
		if (s[i]<pivot) lower[l_ind++]=s[i];
		else {
			if (s[i]>pivot) higher[h_ind++]=s[i];
			else same[s_ind++]=s[i];
		}
	}
	
	quicksort(lower,l_ind);
	quicksort(higher,h_ind);
	
	if (l_ind) for (i=0; i<l_ind; i++)                       s[i] = lower[i];
	if (s_ind) for (i=l_ind; i<s_ind+l_ind; i++)             s[i] = same[i-l_ind];
	if (h_ind) for (i=l_ind+s_ind; i<l_ind+s_ind+h_ind; i++) s[i] = higher[i-l_ind-s_ind];
	
	free(same);
	free(higher);
	free(lower);
	
	return;
}
